local lp, lg = love.physics, love.graphics

local wSizeX, wSizeY = lg.getDimensions()
lg.setLineStyle("rough")

-- Diameter of a billiards ball is 57.15 mm
local ppu = 300
local ballRadius = 0.05715/2*ppu

lp.setMeter(20)
local world = lp.newWorld(0, 0)
local balls = {}
local cushions = {}
local tableSizeX = 2.24*ppu
local tableSizeY = 1.12*ppu
local centrePocketSize = 0.13*ppu
local cornerPocketSize = 0.1155*math.sqrt(0.5)*ppu


local function createBall(x, y)
  local obj = {
    body = lp.newBody(world, x, y, "dynamic");
    shape = lp.newCircleShape(ballRadius);
  }
  local f = lp.newFixture(obj.body, obj.shape)
  obj.fixture = f
  f:setRestitution(1)
  f:setFriction(0.5)
  return obj
end

local function createCushion(x1, y1, x2, y2)
  local centreX, centreY = (x1+x2)/2, (y1+y2)/2
  local obj = {
    body = lp.newBody(world, centreX, centreY, "static");
    shape = lp.newEdgeShape(x1 - centreX, y1 - centreY, x2 - centreX, y2 - centreY);
  }
  local f = lp.newFixture(obj.body, obj.shape)
  obj.fixture = f
  f:setRestitution(1)
  f:setFriction(0.99)
  return obj
end

function love.load()
  local x1, x2, y
  y = tableSizeY/2
  x1 = centrePocketSize/2
  x2 = tableSizeX/2 - cornerPocketSize
  cushions[1] = createCushion(x1, y, x2, y)
  cushions[2] = createCushion(x1, -y, x2, -y)
  cushions[3] = createCushion(-x1, y, -x2, y)
  cushions[4] = createCushion(-x1, -y, -x2, -y)
  x1 = tableSizeX/2
  y = tableSizeY/2 - cornerPocketSize
  cushions[5] = createCushion(x1, -y, x1, y)
  cushions[6] = createCushion(-x1, -y, -x1, y)
  x1 = tableSizeX/4
  balls[1] = createBall(-x1, 0) -- cue ball
  -- There are several ways to build a ball triangle programmatically.
  -- We choose to build it as a rhombus with 60 degrees on one of the corners
  -- and discard all coordinates beyond the centre line. xr and yr are in
  -- rhomb coordinates.
  local idx = 2
  local gapFactor = 1.02  -- 2%
  local xform = love.math.newTransform(x1, 0,
     math.rad(-30),
     2*ballRadius*gapFactor, math.sqrt(3)*ballRadius*gapFactor,
     0, 0,
     0.5, 0)
  for yr = 0, 4 do
    for xr = 0, 4 do
      if xr + yr < 5 then -- within the triangle
        x1, y = xform:transformPoint(xr, yr)
        balls[idx] = createBall(x1, y)
        idx = idx + 1
      end
    end
  end
end



function love.keypressed(k, s)
  if s == "escape" then love.event.quit() return end
  if s == "space" then balls[1].body:applyLinearImpulse(800, 5) end
end

function love.update(dt)
  -- Table friction
  for i = 1, #balls do
    local ball = balls[i].body
    -- Apply linear friction
    local vx, vy = ball:getLinearVelocity()
    local norm = math.sqrt(vx*vx + vy*vy)
    if norm > 0.0001 then
      ball:applyForce(vx/norm*-8, vy/norm*-8)
    else
      ball:setLinearVelocity(0, 0)
    end
    -- Apply rotational friction
    local omega = ball:getAngularVelocity()
    local sign = omega < 0 and -1 or omega > 0 and 1 or 0
    ball:applyTorque(sign*-20)
  end

  world:update(dt)
end

function love.draw()
  lg.translate(wSizeX/2, wSizeY/2)

  for i = 1, #cushions do
    local cushion = cushions[i]
    local x, y = cushion.body:getPosition()
    local x1, y1, x2, y2 = cushion.shape:getPoints()
    x1, y1, x2, y2 = x1 + x, y1 + y, x2 + x, y2 + y
    lg.line(x1, y1, x2, y2)
  end

  for i = 1, #balls do
    local ball = balls[i]
    local x, y = ball.body:getPosition()
    lg.circle("fill", x, y, ballRadius)
    lg.setColor(0, 0.5, 1)
    local angle = ball.body:getAngle()
    lg.line(x, y, x+math.cos(angle)*ballRadius, y+math.sin(angle)*ballRadius)
    lg.setColor(1, 1, 1)
  end
end
